(* Delphi-Unit
   Hinweisfenster fr Status mit Fortschrittsanzeige (opt.)
   ========================================================
   
    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Okt. 2002 
   last modified:  May 2021
   *)

unit StatWind;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Prgrss;

type
  TStatusWindow = class(TForm)
    StatusLabel: TLabel;
    btnCancel: TBitBtn;
    ProgLabel: TLabel;
    pbStatus: TNewProgressBar;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    procedure SetStatus (const Value: string);
    procedure SetInfo (const Value: string);
    procedure SetProgress (Value : integer);
  public
    { Public declarations }
    Stopped : boolean;
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
//    procedure SetState (AState : TProgressBarState);
    procedure ShowStatus (APos       : TPoint;
                          const Titel,Stat : string;
                          AllowBreak : boolean;
                          AInterval  : integer);
    property Status : string write SetStatus;
    property Info : string write SetInfo;
    property Progress : integer write SetProgress;
  end;

function CreateStatusWindow (APos : TPoint; const Titel,Stat : string;
                      AllowBreak : boolean; AInterval  : integer) : TStatusWindow;

procedure ReleaseStatusWindow (StatusWindow : TStatusWindow);

var
  StatusWindow: TStatusWindow;

implementation

{$R *.dfm}

uses Vcl.FileCtrl, GgtDummy, WinUtils;

{ ---------------------------------------------------------------- }
procedure TStatusWindow.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TStatusWindow.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TStatusWindow.FormResize(Sender: TObject);
var
  w : integer;
begin
  w:=ClientWidth;
  with btnCancel do Left:=(w-Width) div 2;
  end;

procedure TStatusWindow.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  end;

{ ---------------------------------------------------------------- }
procedure TStatusWindow.btnCancelClick(Sender: TObject);
begin
  Stopped:=true;
  end;

//procedure TStatusWindow.SetState (AState : TProgressBarState);
//begin
//  pbStatus.State:=AState;
//  end;

procedure TStatusWindow.SetStatus (const Value : string);
begin
  with StatusLabel do Caption:=MinimizeName(Value,Canvas,pbStatus.Width);
  end;

procedure TStatusWindow.SetInfo (const Value : string);
begin
  with ProgLabel do Caption:=MinimizeName(Value,Canvas,pbStatus.Width);
  end;

procedure TStatusWindow.SetProgress (Value : integer);
begin
  with pbStatus do begin
    if Value<Min then Value:=Min;
    if Value>Max then Value:=Max;
    Position:=Value;
    end;
  end;

{ ---------------------------------------------------------------- }
(* String in Hinweisfenster bei X,Y anzeigen *)
procedure TStatusWindow.ShowStatus (APos       : TPoint;
                                    const Titel,Stat : string;
                                    AllowBreak : boolean;
                                    AInterval  : integer);
(* Titel  : Fensterberschrift
   Stat   : anzuzeigender String
   AInterval  > 0 : Fortschrittsbalken im Marquee-Stil
              < 0 : normaler Fortschrittsbalken
              = 0 : kein Balken *)
var
  Neu  : boolean;
begin
  Neu:=not Visible;
  Stopped:=false;
  if AllowBreak then begin
    ClientHeight:=btnCancel.Top+btnCancel.Height+5;
    btnCancel.Show;
    end
  else begin
    ClientHeight:=btnCancel.Top;
    btnCancel.Hide;
    end;
  if Neu or (Titel<>'') then Caption:=Titel;
  ProgLabel.Caption:='';
  StatusLabel.Caption:=Stat;
  if Neu then begin
    with pbStatus do begin
      if AInterval>0 then begin
        Style:=npsContinuous;
        Visible:=true; Interval:=AInterval;
        end
      else if AInterval<0 then begin
        Style:=npsNormal;
        Position:=0;
        end
      else Visible:=false;
      end;
    AdjustFormPosition(Screen,self,APos);
    btnCancel.Left:=(ClientWidth-btnCancel.Width) div 2;
    end;
  if Neu then Show else Update;
  end;

function CreateStatusWindow (APos : TPoint; const Titel,Stat : string;
                      AllowBreak : boolean; AInterval  : integer) : TStatusWindow;
begin
  Result:=TStatusWindow.Create(Application);
  with Result do begin
    FormStyle:=fsStayOnTop;
    ShowStatus(APos,Titel,Stat,AllowBreak,AInterval);
    end;
  Application.ProcessMessages;
  end;

procedure ReleaseStatusWindow (StatusWindow : TStatusWindow);
begin
  with StatusWindow do begin
    Close; Release;
    end;
  StatusWindow:=nil;
  end;

end.
